#ifndef SERIAL_H_
#define SERIAL_H_

void Serial_init(void);
void Serial_transmit(uint8_t data);
uint8_t Serial_receive(void);
int Serial_receive_timeout(uint8_t *data, int timeout);
void Serial_write(const char *string);
void Serial_writeln(const char *string);
void Serial_read(char *string, int length);
void Serial_read_string(char *string);
int Serial_available(void);
void Serial_find(const char *string);
int Serial_find_timeout(const char *string, int timeout);

#endif /* SERIAL_H_ */